/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	do_load		handle loading Casio data			*/
/*									*/
/************************************************************************/

#include	<stdio.h>
#include	<sys/param.h>
#include	<sys/types.h>
#include	<xview/xview.h>
#include	<xview/panel.h>
#include	"boss_ui.h"

#include	"manifest.h"
#include	"object.h"
#include	"globals.h"

#define		FROM_BOSS		0
#define		FROM_CASIO_FILE		1
#define		FROM_TEXT_FILE		2

PUBLIC	boss_base_objects	*boss_base;

PRIVATE	boss_load_dialog_objects	*dialog = NULL;

/************************************************************************/
EXPORT	Menu_item	load_dialog(item, op)

Menu_item	item;
Menu_generate	op;

{
	if (op == MENU_NOTIFY) {
	   if (dialog == NULL) {
	      dialog = boss_load_dialog_objects_initialize(NULL, boss_base->base);
	      xv_set(dialog->casio_source,
	                PANEL_VALUE, casio_file,
	                PANEL_NOTIFY_LEVEL, PANEL_SPECIFIED,
	                PANEL_NOTIFY_STRING, "\n\r\t ",
	                PANEL_INACTIVE, TRUE,
	             NULL);
	      xv_set(dialog->ascii_source,
	                PANEL_NOTIFY_LEVEL, PANEL_SPECIFIED,
	                PANEL_NOTIFY_STRING, "\n\r\t ",
	                PANEL_INACTIVE, TRUE,
	             NULL);
	      xv_set(dialog->source, PANEL_VALUE, 0, NULL);
	      place_dialog(boss_base->base, dialog->load_dialog);
	      }
	   xv_set(dialog->load_dialog, XV_SHOW, TRUE, NULL);
	   }
	return item;
}

/************************************************************************/
EXPORT	void	load_notify(item, value, event)

Panel_item	item;
int		value;
Event		*event;

{
	if (value == 0) {
	   xv_set(dialog->casio_source, PANEL_INACTIVE, TRUE, NULL);
	   xv_set(dialog->ascii_source, PANEL_INACTIVE, TRUE, NULL);
	   }
	else if (value == 1) {
	   xv_set(dialog->casio_source, PANEL_INACTIVE, FALSE, NULL);
	   xv_set(dialog->ascii_source, PANEL_INACTIVE, TRUE, NULL);
	   xv_set(dialog->load_control, PANEL_CARET_ITEM, dialog->casio_source, NULL);
	   }
	else {
	   xv_set(dialog->casio_source, PANEL_INACTIVE, TRUE, NULL);
	   xv_set(dialog->ascii_source, PANEL_INACTIVE, FALSE, NULL);
	   xv_set(dialog->load_control, PANEL_CARET_ITEM, dialog->ascii_source, NULL);
	   }
}

/************************************************************************/
EXPORT	void	accept_load(item, event)

Panel_item	item;
Event		*event;

{	int	curr, len;
	char	*p, buf[MAX_OBJECT_DATA], line[1024];
	FILE	*f;
	object	*o, *obj;

	lets_get_busy(boss_base->base, TRUE, NULL);
	xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, 0);

	if (((int) xv_get(dialog->source, PANEL_VALUE)) == FROM_BOSS) {
	   if (open_casio(port_file, baud_rate, parity)) {
	      while (obj = receive_object())
	         add_to_list(obj);
	      close_casio();
	      xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
	      }
	   }
	else if (((int) xv_get(dialog->source, PANEL_VALUE)) == FROM_CASIO_FILE) {
	   p = (char *) xv_get(dialog->casio_source, PANEL_VALUE);
	   if (*p == '\0')
	      error("You must specify a Casio-format file to be read");
	   else if (lex_init(p)) {
	      parse_errors_occured = FALSE;
	      yyparse();
	      if (!parse_errors_occured) {
	         for (obj = parsed_object_list; obj; ) {
	            o = obj;
	            obj = obj->next;
	            add_to_list(o);
	            }
	         xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
	         }
	      }
	   else
	      error("Could not read from %s", p);
	   }
	else if (((int) xv_get(dialog->source, PANEL_VALUE)) == FROM_TEXT_FILE) {
	   p = (char *) xv_get(dialog->ascii_source, PANEL_VALUE);
	   if (*p == '\0')
	      error("You must specify a ASCII-format file to be read");
	   else if (f = fopen(p, "r")) {
	      curr = 0;
	      while (fgets(line, 512, f)) {
	         if (curr + (len = strlen(line)) >= MAX_OBJECT_DATA) {
	            add_to_list(text_to_object(buf));
	            curr = 0;
	            }
	         while (len >= MAX_OBJECT_DATA) {
	            strncpy(buf, line, MAX_OBJECT_DATA - 1);
	            add_to_list(text_to_object(buf));
	            strcpy(line, line + MAX_OBJECT_DATA - 1);
	            len -= MAX_OBJECT_DATA - 1;
	            }
	         strcpy(buf + curr, line);
	         curr += len;
	         }
	      if (curr > 0)
	         add_to_list(text_to_object(buf));
	      fclose(f);
	      xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
	      }
	   else
	      error("Could not read from %s", p);
	   }
	lets_get_busy(boss_base->base, FALSE, NULL);
}
