/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	do_store	Handle storing Casio data			*/
/*									*/
/************************************************************************/

#include	<stdio.h>
#include	<sys/param.h>
#include	<sys/types.h>
#include	<xview/xview.h>
#include	<xview/panel.h>
#include	<xview/notice.h>
#include	"boss_ui.h"

#include	"manifest.h"
#include	"packet.h"
#include	"object.h"
#include	"globals.h"

#define		TO_BOSS			0
#define		TO_CASIO_FILE		1
#define		TO_TEXT_FILE		2

PUBLIC	boss_base_objects	*boss_base;

PRIVATE	boss_store_dialog_objects	*dialog = NULL;

/************************************************************************/
EXPORT	Menu_item	store_dialog(item, op)

Menu_item	item;
Menu_generate	op;

{
	if (op == MENU_NOTIFY) {
	   if (dialog == NULL) {
	      dialog = boss_store_dialog_objects_initialize(NULL, boss_base->base);
	      xv_set(dialog->casio_dest,
	                PANEL_VALUE, casio_file,
	                PANEL_NOTIFY_LEVEL, PANEL_SPECIFIED,
	                PANEL_NOTIFY_STRING, "\n\r\t ",
	                PANEL_INACTIVE, TRUE,
	             NULL);
	      xv_set(dialog->ascii_dest,
	                PANEL_NOTIFY_LEVEL, PANEL_SPECIFIED,
	                PANEL_NOTIFY_STRING, "\n\r\t ",
	                PANEL_INACTIVE, TRUE,
	             NULL);
	      xv_set(dialog->store_data, PANEL_VALUE, (1 << MAX_OBJECT_KINDS) - 1, NULL);
	      xv_set(dialog->dest, PANEL_VALUE, 0, NULL);
	      place_dialog(boss_base->base, dialog->store_dialog);
	      }
	   xv_set(dialog->store_dialog, XV_SHOW, TRUE, NULL);
	   }
	return item;
}

/************************************************************************/
EXPORT	void	store_notify(item, value, event)

Panel_item	item;
int		value;
Event		*event;

{
	if (value == 0) {
	   xv_set(dialog->casio_dest, PANEL_INACTIVE, TRUE, NULL);
	   xv_set(dialog->ascii_dest, PANEL_INACTIVE, TRUE, NULL);
	   }
	else if (value == 1) {
	   xv_set(dialog->casio_dest, PANEL_INACTIVE, FALSE, NULL);
	   xv_set(dialog->ascii_dest, PANEL_INACTIVE, TRUE, NULL);
	   xv_set(dialog->store_control, PANEL_CARET_ITEM, dialog->casio_dest, NULL);
	   }
	else {
	   xv_set(dialog->casio_dest, PANEL_INACTIVE, TRUE, NULL);
	   xv_set(dialog->ascii_dest, PANEL_INACTIVE, FALSE, NULL);
	   xv_set(dialog->store_control, PANEL_CARET_ITEM, dialog->ascii_dest, NULL);
	   }
}

/************************************************************************/
EXPORT	void	accept_store(item, event)

Panel_item	item;
Event		*event;

{	int	i, which;
	FILE	*f;
	object	*obj;
	char	*path;

	lets_get_busy(boss_base->base, TRUE, NULL);
	xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, 0);

	if ((which = (int) xv_get(dialog->store_data, PANEL_VALUE)) == 0)
	   xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
	else if (((int) xv_get(dialog->dest, PANEL_VALUE)) == TO_BOSS) {
	   if (baud_rate == BAUD_9600) {
	      i = notice_prompt(boss_base->base, NULL,
	                           NOTICE_MESSAGE_STRINGS, "Storing data to the BOSS at 9600 baud is known",
	                           			   "to be unreliable.  You should reduce the baud",
	                             			   "rate to 4800 or lower before proceeding.",
	                             			   " ",
	                             			   "Do you wish to continue?",
	                             			   NULL,
	                           NOTICE_BUTTON_YES, "Continue",
	                           NOTICE_BUTTON_NO, "Cancel",
	                        0);
	      if (i == NOTICE_NO) {
	         lets_get_busy(boss_base->base, FALSE, NULL);
	         return;
	         }
	      }
	   if (open_casio(port_file, baud_rate, parity)) {
	      if (start_session()) {
	         for (i = 0; i < MAX_OBJECT_KINDS; i++)
	            if (which & (1 << i))
	               for (obj = data[i]; obj; obj = obj->next)
	                  if (!obj->from_casio)
	                     if (!send_object(obj))
	                        break;
	         end_session();
	         if (obj == NULL)
	            xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
	         }
	      close_casio();
	      }
	   }
	else if (((int) xv_get(dialog->dest, PANEL_VALUE)) == TO_CASIO_FILE) {
	   path = (char *) xv_get(dialog->casio_dest, PANEL_VALUE);
	   if (*path == '\0')
	      error("You must specify a file to receive the Casio-format data");
	   else if ((f = fopen(path, "w")) == NULL)
	      error("cannot write to %s", path);
	   else {
	      for (i = 0; i < MAX_OBJECT_KINDS; i++)
	         if (which & (1 << i))
	            for (obj = data[i]; obj; obj = obj->next)
	               print_object_internal(f, obj);
	      fclose(f);
	      xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
	      }
	   }
	else if (((int) xv_get(dialog->dest, PANEL_VALUE)) == TO_TEXT_FILE) {
	   path = (char *) xv_get(dialog->ascii_dest, PANEL_VALUE);
	   if (*path == '\0')
	      error("You must specify a file to receive the ASCII-format data");
	   else if ((f = fopen(path, "w")) == NULL)
	      error("cannot write to %s", path);
	   else {
	      for (i = 0; i < MAX_OBJECT_KINDS; i++)
	         if (which & (1 << i))
	            for (obj = data[i]; obj; obj = obj->next)
	               print_object_ascii(f, obj);
	      fclose(f);
	      xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
	      }
	   }
	lets_get_busy(boss_base->base, FALSE, NULL);
}
