/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	print.c	handle text output of objects				*/
/*									*/
/************************************************************************/

#include	<stdio.h>
#include	<ctype.h>

#include	"manifest.h"
#include	"object.h"

PRIVATE	char	*month[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

/************************************************************************/
PRIVATE	internal_date(f, d)

FILE	*f;
date	*d;

{
	fprintf(f, "   {date\t%d/%02d/%4d}\n", d->month, d->day, d->year);
}

/************************************************************************/
PRIVATE	internal_time(f, name, t)

FILE	*f;
char	*name;
time	*t;

{
	if (t->hour != INVALID_TIME)
	   fprintf(f, "   {%s\t%2d:%02d}\n", name, t->hour, t->minute);
}

/************************************************************************/
PRIVATE	internal_attribute(f, name, field)

FILE	*f;
char	*name;
byte	*field;

{
	if (field && *field) {
	   fprintf(f, "   {%s\t\"", name);
	   for ( ; *field; field++)
	      if (*field == '"')
	         fprintf(f, "\\\"");
	      else if (*field >= '\177')
	         fprintf(f, "\\%03o", *field);
	      else if (iscntrl(*field))
	         fprintf(f, "^%c", *field + '@');
	      else
	         fprintf(f, "%c", *field);
	   fprintf(f, "\"}\n");
	   }
}

/************************************************************************/
PRIVATE	print_date(f, d)

FILE	*f;
date	*d;

{
	fprintf(f, "%s %d, %d ", month[d->month - 1], d->day, d->year);
}

/************************************************************************/
PRIVATE	print_time(f, leader, t)

FILE	*f;
char	*leader;
time	*t;

{
	if (t->hour != INVALID_TIME)
	   fprintf(f, "%s%2d:%02d", leader, t->hour, t->minute);
}

/************************************************************************/
PRIVATE	print_string(f, str)

FILE	*f;
byte	*str;

{
	if (str && *str) {
	   for ( ; *str; str++)
	      if (*str == '\r')
	         fprintf(f, "\n");
	      else if (*str >= '\177')
	         fprintf(f, "?");
	      else if (iscntrl(*str))
	         fprintf(f, "^%c", *str + '@');
	      else
	         fprintf(f, "%c", *str);
	   fprintf(f, "\n");
	   }
}

/************************************************************************/
PRIVATE	print_pair(f, p, q)

FILE	*f;
char	*p;
char	*q;

{	int	i;

	if (p == NULL)
	   p = "";
	if (q == NULL)
	   q = "";
	fprintf(f, "%s%*s%s\n", p, 32 - strlen(p) - strlen(q), " ", q);
}

/************************************************************************/
EXPORT	print_object_internal(f, obj)

FILE	*f;
object	*obj;

{	int	i;

	fprintf(f, "{");
	if (obj->kind == OBJ_TELEPHONE) {
	   fprintf(f, "telephone\n");
	   internal_attribute(f, "name",    obj->pho_name);
	   internal_attribute(f, "number",  obj->pho_number);
	   internal_attribute(f, "address", obj->pho_address);
	   internal_attribute(f, "extra_1", obj->pho_extra_1);
	   internal_attribute(f, "extra_2", obj->pho_extra_2);
	   internal_attribute(f, "extra_3", obj->pho_extra_3);
	   internal_attribute(f, "extra_4", obj->pho_extra_4);
	   internal_attribute(f, "extra_5", obj->pho_extra_5);
	   internal_attribute(f, "extra_6", obj->pho_extra_6);
	   }
	else if (obj->kind == OBJ_SCHEDULE) {
	   fprintf(f, "schedule\n");
	   internal_date(f, &obj->sch_date);
	   internal_time(f, "start", &obj->sch_start_time);
	   internal_time(f, "stop", &obj->sch_stop_time);
	   internal_time(f, "alarm", &obj->sch_alarm_time);
	   internal_attribute(f, "memo", obj->sch_memo);
	   }
	else if (obj->kind == OBJ_MEMO) {
	   fprintf(f, "memo\n");
	   internal_attribute(f, "memo", obj->mem_memo);
	   }
	else if (obj->kind == OBJ_CARD) {
	   fprintf(f, "card\n");
	   internal_attribute(f, "employer",   obj->car_employer);
	   internal_attribute(f, "name",       obj->car_name);
	   internal_attribute(f, "number",     obj->car_number);
	   internal_attribute(f, "position",   obj->car_position);
	   internal_attribute(f, "department", obj->car_department);
	   internal_attribute(f, "pobox",      obj->car_pobox);
	   internal_attribute(f, "address",    obj->car_address);
	   internal_attribute(f, "telex",      obj->car_telex);
	   internal_attribute(f, "fax",        obj->car_fax);
	   internal_attribute(f, "extra_1",    obj->car_extra_1);
	   internal_attribute(f, "extra_2",    obj->car_extra_2);
	   internal_attribute(f, "extra_3",    obj->car_extra_3);
	   internal_attribute(f, "extra_4",    obj->car_extra_4);
	   internal_attribute(f, "extra_5",    obj->car_extra_5);
	   internal_attribute(f, "extra_6",    obj->car_extra_6);
	   }
	else if (obj->kind == OBJ_CALENDAR) {
	   fprintf(f, "calendar\n");
	   internal_date(f, &obj->cal_date);
	   if (obj->cal_marked) {
	      fprintf(f, "   {highlight");
	      for (i = 1; i <= 31; i++)
	         if (obj->cal_marked & (1 << (i - 1)))
	            fprintf(f, " %d", i);
	      fprintf(f, "}\n");
	      }
	   }
	else
	   error("unknown object passed to print_object_internal (%d)", obj->kind);
	if (obj->marked)
	   fprintf(f, "   {marked}\n");
	fprintf(f, "}\n");
}

/************************************************************************/
EXPORT	print_object_ascii(f, obj)

FILE	*f;
object	*obj;

{	int	i;

	if (obj->kind == OBJ_TELEPHONE) {
	   print_pair(f, obj->pho_name, obj->pho_number);
	   print_string(f, obj->pho_address);
	   print_string(f, obj->pho_extra_1);
	   print_string(f, obj->pho_extra_2);
	   print_string(f, obj->pho_extra_3);
	   print_string(f, obj->pho_extra_4);
	   print_string(f, obj->pho_extra_5);
	   print_string(f, obj->pho_extra_6);
	   }
	else if (obj->kind == OBJ_SCHEDULE) {
	   print_date(f, &obj->sch_date);
	   print_time(f, " ", &obj->sch_start_time);
	   print_time(f, "-", &obj->sch_stop_time);
	   print_time(f, " @", &obj->sch_alarm_time);
	   fprintf(f, "\n");
	   print_string(f, obj->sch_memo);
	   }
	else if (obj->kind == OBJ_MEMO) {
	   print_string(f, obj->mem_memo);
	   }
	else if (obj->kind == OBJ_CARD) {
	   print_pair(f, obj->car_name, obj->car_number);
	   print_string(f, obj->car_employer);
	   print_string(f, obj->car_position);
	   print_string(f, obj->car_department);
	   print_string(f, obj->car_pobox);
	   print_string(f, obj->car_address);
	   print_string(f, obj->car_telex);
	   print_string(f, obj->car_fax);
	   print_string(f, obj->car_extra_1);
	   print_string(f, obj->car_extra_2);
	   print_string(f, obj->car_extra_3);
	   print_string(f, obj->car_extra_4);
	   print_string(f, obj->car_extra_5);
	   print_string(f, obj->car_extra_6);
	   }
	else if (obj->kind == OBJ_CALENDAR) {
	   fprintf(f, "%s %d:", month[obj->cal_date.month - 1], obj->cal_date.year);
	   for (i = 1; i <= 31; i++)
	      if (obj->cal_marked & (1 << (i - 1)))
	         fprintf(f, " %d", i);
	   fprintf(f, "\n");
	   }
	else
	   error("unknown object passed to print_object_ascii (%d)", obj->kind);
	fprintf(f, "\n");
}
