/************************************************************************/
/*	Copyright 1987-1991 by Chuck Musciano and Harris Corporation 	*/
/*									*/
/*	Full ownership of this software, and all rights pertaining to 	*/
/*	the for-profit distribution of this software, are retained by 	*/
/*	Chuck Musciano and Harris Corporation.  You are permitted to 	*/
/*	use this software without fee.  This software is provided "as 	*/
/*	is" without express or implied warranty.  You may redistribute 	*/
/*	this software, provided that this copyright notice is retained,	*/
/*	and that the software is not distributed for profit.  If you 	*/
/*	wish to use this software in a profit-making venture, you must 	*/
/*	first license this code and its underlying technology from 	*/
/*	Harris Corporation. 						*/
/*									*/
/*	Bottom line: you can have this software, you can use it, you 	*/
/*	can give it away.  You just can't sell any or all parts of it 	*/
/*	without prior permission from Harris Corporation. 		*/
/************************************************************************/

/************************************************************************/
/*									*/
/*	send_mail.c	display a little information window		*/
/*									*/
/************************************************************************/

#include	<stdio.h>
#include	<sys/param.h>
#include	<sys/types.h>
#include	<xview/xview.h>
#include	<xview/panel.h>
#include	<xview/textsw.h>
#include	"boss_ui.h"

#include	"manifest.h"

#define		VERSION			"1.0b"

PUBLIC	boss_base_objects	*boss_base;

PRIVATE	boss_mail_objects	*dialog = NULL;

/************************************************************************/
EXPORT	popup_send_mail(item, event)

Panel_item	item;
Event		*event;

{
	if (dialog == NULL) {
	   dialog = boss_mail_objects_initialize(NULL, boss_base->base);
	   place_dialog(boss_base->base, dialog->mail);
	   xv_set(dialog->other_address, WIN_SHOW, FALSE, NULL);
	   }
	xv_set(dialog->mail, XV_SHOW, TRUE, NULL);
	xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
}

/************************************************************************/
EXPORT	void	handle_address(item, value, event)

Panel_item	item;
int		value;
Event		*event;

{
	if (value == 2) {
	   xv_set(dialog->other_address, WIN_SHOW, TRUE, NULL);
	   panel_paint(dialog->address, PANEL_NO_CLEAR);
	   panel_paint(dialog->other_address, PANEL_NO_CLEAR);
	   }
	else {
	   xv_set(dialog->other_address, WIN_SHOW, FALSE, NULL);
	   panel_paint(dialog->address, PANEL_NO_CLEAR);
	   }
}

/************************************************************************/
EXPORT	void	done_mail(frame)

Frame		frame;

{
	textsw_reset(dialog->message, 0, 0);
	xv_set(frame, XV_SHOW, FALSE, 0);
}

/************************************************************************/
EXPORT	void	send_mail(item, event)

Panel_item	item;
Event		*event;

{	char	cmd[1024], *buf;
	int	val, size;
	FILE	*pipe;

	xv_set(item, PANEL_NOTIFY_STATUS, XV_ERROR, 0);
	val = (int) xv_get(dialog->address, PANEL_VALUE);
	if (val != 2)
	   sprintf(cmd, "%s %s", MAILER, xv_get(dialog->address, PANEL_CHOICE_STRING, val));
	else {
	   buf = (char *) xv_get(dialog->other_address, PANEL_VALUE);
	   if (is_empty(buf)) {
	      error("You must specify an alternate address before sending your message");
	      return;
	      }
	   else
	      sprintf(cmd, "%s %s", MAILER, buf);
	   }
	size = (int) xv_get(dialog->message, TEXTSW_LENGTH);
	if (size == 0) {
	   error("Please type a message before sending the mail");
	   return;
	   }
	if ((pipe = popen(cmd, "w")) == NULL) {
	   error("Could not invoke \"%s\" to send mail", cmd);
	   return;
	   }
	lets_get_busy(boss_base->base, TRUE, NULL);
	buf = (char *) malloc(size);
	xv_get(dialog->message, TEXTSW_CONTENTS, 0, buf, size);
	fprintf(pipe, "Subject: Boss %s comment\n\n", VERSION);
	if (fwrite(buf, 1, size, pipe) != size) {
	   error("Could not write message to the mailer");
	   free(buf);
	   return;
	   }
	pclose(pipe);
	free(buf);
	textsw_reset(dialog->message, 0, 0);
	lets_get_busy(boss_base->base, FALSE, NULL);
	xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
}

/************************************************************************/
EXPORT	void	cancel_mail(item, event)

Panel_item	item;
Event		*event;

{
	textsw_reset(dialog->message, 0, 0);
	xv_set(item, PANEL_NOTIFY_STATUS, XV_OK, 0);
}
